/************************************************************************
*                                                                       *
*       gimp_gradient.c                                                 *
*       This file contains the code necessary to perform a grayscale    *
*       dilation on a image for a GIMP plug-in.                         *
*                                                                       *
*       Author: Bryan Evenson                                           *
*                                                                       *
*               -Ideas for the GIMP plug-in specific parts of the       *
*               code liberally borrowed from files at                   *
*               http:/cvs.gnome.org.                                    *
*                                                                       *
*                                                                       *
*       Created: May 16, 1999                                           *
*                                                                       *
*       Revision 1.0                                                    *
*                                                                       *
************************************************************************/

#include <stdlib.h>
#include <libgimp/gimp.h>
#include "gimp_morph.h"


GPixelRgn gimp_gradient(GDrawable *drawable, int Se[5][5], int Px, int Py)
{
        GPixelRgn srcPR, dil_destPR, er_destPR, destPR;
        gint width, height;
        gint bytes;
        guchar *dil_src, *er_src, *dest;
        gint row, col;
        gint x1, y1, x2, y2;


        /* Get the input area. This is the bounding box of the selection in
        *  the image (or the entire image if there is no selection). Only
        *  operating on the input area is simply an optimization. It doesn't
        *  need to be done for correct operation. (It simply makes it go
        *  faster, since fewer pixels need to be operated on).
        */
        gimp_drawable_mask_bounds (drawable->id, &x1, &y1, &x2, &y2);

        /* Get the size of the input image. (This will/must be the same
         *  as the size of the output image.
         */
        width = drawable->width;
        height = drawable->height;
        bytes = drawable->bpp;

        /*  allocate row buffers  */
        dil_src = (guchar *) malloc ((x2 - x1) * bytes);
        er_src = (guchar *) malloc ((x2 - x1) * bytes);
        dest = (guchar *) malloc ((x2 - x1) * bytes);


        /*  initialize the pixel regions  */
        gimp_pixel_rgn_init (&srcPR, drawable, 0, 0, width, height, FALSE, FALSE);
	gimp_pixel_rgn_init (&er_destPR, drawable, 0, 0, width, height, FALSE, FALSE);
	gimp_pixel_rgn_init (&dil_destPR, drawable, 0, 0, width, height, FALSE, FALSE);
	gimp_pixel_rgn_init (&destPR, drawable, 0, 0, width, height, TRUE, TRUE);

       
	/*Perform the dilation and the erosion.  Get the results. */
	dil_destPR = gimp_dilate(drawable, Se, Px, Py);
	er_destPR = gimp_erode(drawable, Se, Px, Py);
	
        gimp_progress_init ("Subtracting erosion from dilation...");
	
	/* Subtract the erosion of the image from the dilation. */
	for(row=y1; row<y2; row++)
	{
		/* Get a row of data from each the dilation and the erosion. */
		gimp_pixel_rgn_get_row(&dil_destPR, dil_src, x1, row, (x2 - x1));
		gimp_pixel_rgn_get_row(&er_destPR, er_src, x1, row, (x2 - x1));

		/* Perform the subtraction. */
		for(col=x1; col<x2; col++)
			dest[col*bytes] = abs((gint)dil_src[col*bytes] - (gint)er_src[col*bytes]);

		/* Store the data. */
		gimp_pixel_rgn_set_row(&destPR, dest, x1, row, (x2 - x1));

		/* Update the progress meter. */
		if((row%5)==0)
			gimp_progress_update((double) row/(double) (y2 - y1));
	}

        free (dil_src);
        free (er_src);
        free (dest);

	return destPR;
}
