/***********************************************************************

	File:			ui.c
	Author:		Ted Gould
	Date:			Thu May  6 00:48:02 EST 1999

	This file will take care of the "GIMP side" of the plug-in.  It
	builds the GUI and also interpets the stuff that is comming from the
	GIMP at large.  Some of this stuff is dealing with the interactive
	and non-interactive stuff in the connection and pulling up old values.

***********************************************************************/

/* System Libs */
#include <stdlib.h>
#include <string.h>
#include <math.h>

/* GIMP and GTK libs */
#include <libgimp/gimp.h>
#include <gtk/gtk.h>

/* Local Libs */
#include "ui.h"
#include "gimp_morph.h"


GDrawable * global_drawable;
int global_matrix[INPUT_MATRIX_SIZE][INPUT_MATRIX_SIZE];
int global_center_x, global_center_y;
GPixelRgn (* dofunc)(GDrawable *drawable, int Se[][], int Px, int Py);
int went_okay;
int origin_x1, origin_x2, origin_y1, origin_y2;

MAIN()

static void query (void) {

	static GParamDef params[] = {
		{	PARAM_INT32,		"run_mode",		"Interactive or non-Interactive"},
		{	PARAM_IMAGE,		"image_id",		"(unused)"},
		{	PARAM_DRAWABLE,	"drawable_id",	"This is somplace to draw"}
		};

	gimp_install_procedure(	PLUG_IN_NAME_DILATE,
									"A grey-scale morphological dilation.",
									"To use this function what you really need to do is have lots of processor time to waste",
									"Ted Gould and Bryan Evenson",
									"GPL",
									"Today",
									"<Image>/Filters/Image Processing/Dilate",
									"GRAY",
									PROC_PLUG_IN,
									3,
									0,
									params,
									NULL);

	gimp_install_procedure(	PLUG_IN_NAME_ERODE,
									"A grey-scale morphological erotion.",
									"To use this function what you really need to do is have lots of processor time to waste",
									"Ted Gould and Bryan Evenson",
									"GPL",
									"Today",
									"<Image>/Filters/Image Processing/Erode",
									"GRAY",
									PROC_PLUG_IN,
									3,
									0,
									params,
									NULL);

	gimp_install_procedure(	PLUG_IN_NAME_OPENING,
									"A grey-scale morphological opening.",
									"To use this function what you really need to do is have lots and lots of processor time to waste",
									"Ted Gould and Bryan Evenson",
									"GPL",
									"Today",
									"<Image>/Filters/Image Processing/Opening",
									"GRAY",
									PROC_PLUG_IN,
									3,
									0,
									params,
									NULL);

	gimp_install_procedure(	PLUG_IN_NAME_CLOSING,
									"A grey-scale morphological closing.",
									"To use this function what you really need to do is have lots and lots of processor time to waste",
									"Ted Gould and Bryan Evenson",
									"GPL",
									"Today",
									"<Image>/Filters/Image Processing/Closing",
									"GRAY",
									PROC_PLUG_IN,
									3,
									0,
									params,
									NULL);

	/*
	gimp_install_procedure(	PLUG_IN_NAME_GRADIENT,
									"A grey-scale morphological gradient.",
									"To use this function what you really need to do is have lots and lots of processor time to waste",
									"Ted Gould and Bryan Evenson",
									"GPL",
									"Today",
									"<Image>/Filters/Image Processing/Gradient",
									"GRAY",
									PROC_PLUG_IN,
									3,
									0,
									params,
									NULL);
*/

	return;
	} /* query */

static void run (gchar * name,
						gint	nparams,
						GParam	* param,
						gint	* nreturn_vals,
						GParam	**	return_vals) {
	static GParam values[1];
	GStatusType status = STATUS_SUCCESS;

	int argc = 1;
	char ** argv;

	argv = (char **)malloc(sizeof(char *));
	argv = &name;

	gtk_init(&argc, &argv);
	free(argv);
	/* gimp_tile_cache_ntiles (2 * (MAX (global_drawable->width, global_drawable->height) * (gimp_tile_width () + 1)));  */

	gtk_rc_parse(gimp_gtkrc());

	default_global_values(param[2]);
	get_gimp_saved_values();
	went_okay = 0;

	dofunc = solve_which_function(name);

	morph_create_window();

	gtk_main();

	values[0].type = PARAM_STATUS;
	values[0].data.d_status = status;

	*nreturn_vals = 1;
	*return_vals = values;

	if (went_okay) {
		put_gimp_saved_values();
		}

	gimp_drawable_flush(global_drawable);
	gimp_drawable_merge_shadow(global_drawable->id, TRUE);
	gimp_drawable_update(global_drawable->id, origin_x1, origin_y1, (origin_x2 - origin_x1), (origin_y2 - origin_y1));

	gimp_drawable_detach(global_drawable);
	gimp_displays_flush();
	} /* run */

GPixelRgn (* solve_which_function (char * name))(GDrawable *drawable, int Se[][], int Px, int Py) {
	if (!g_strcasecmp(name, PLUG_IN_NAME_DILATE)) {
		return gimp_dilate;
		}
	if (!g_strcasecmp(name, PLUG_IN_NAME_ERODE)) {
		return gimp_erode;
		}
	if (!g_strcasecmp(name, PLUG_IN_NAME_OPENING)) {
		return open_me_up;
		}
	if (!g_strcasecmp(name, PLUG_IN_NAME_CLOSING)) {
		return close_me_down;
		}
	if (!g_strcasecmp(name, PLUG_IN_NAME_GRADIENT)) {
		return gimp_gradient;
		}

	g_print("Uh... I don't know what you are doing.\n");
	return gimp_dilate;
	} /* solve_which_function */

void hello( GtkWidget *widget, gpointer data ) {
	gimp_drawable_mask_bounds(global_drawable->id, &origin_x1, &origin_y1, &origin_x2, &origin_y2);
	(*dofunc)(global_drawable, global_matrix, global_center_x, global_center_y);
	went_okay = 1;
	return;
	} /* hello */

void get_gimp_saved_values (void) {
	/* This function will eventually work to pull saved values from the gimp database... */
	gint sizeofmask = -1;
	gint x, *tempmask;
	gint squarert;

	gimp_get_data(PLUG_IN_SAVED_CENTER_X, (gpointer)&global_center_x);
	gimp_get_data(PLUG_IN_SAVED_CENTER_Y, (gpointer)&global_center_y);
	gimp_get_data(PLUG_IN_SAVED_SIZE_OF_MATRIX, (gpointer)&sizeofmask);

	if (sizeofmask > 0) {
		squarert = (int)sqrt(sizeofmask);
		tempmask = (gint *)malloc(sizeofmask * sizeof(gint));
		gimp_get_data(PLUG_IN_SAVED_MASK, (gpointer)tempmask);
		for (x = 0; x < sizeofmask; x ++) {
			global_matrix[x / squarert][x % squarert] = tempmask[x];
			} /* for */
		free(tempmask);
		}

	return;
	} /* get_gimp_saved_values */

void put_gimp_saved_values (void) {
	/* This is where the values get put into the gimp data area. */
	gint x, tempx, tempmaskarray[INPUT_MATRIX_SIZE * INPUT_MATRIX_SIZE];
	tempx = INPUT_MATRIX_SIZE * INPUT_MATRIX_SIZE;

	gimp_set_data(PLUG_IN_SAVED_CENTER_X, (gpointer)&global_center_x, sizeof(gint));
	gimp_set_data(PLUG_IN_SAVED_CENTER_Y, (gpointer)&global_center_y, sizeof(gint));
	gimp_set_data(PLUG_IN_SAVED_SIZE_OF_MATRIX, (gpointer)&tempx, sizeof(gint));

	for (x = 0; x < tempx; x++) {
		tempmaskarray[x] = global_matrix[x / INPUT_MATRIX_SIZE][x % INPUT_MATRIX_SIZE];
		}

	gimp_set_data(PLUG_IN_SAVED_MASK, (gpointer)tempmaskarray, INPUT_MATRIX_SIZE * INPUT_MATRIX_SIZE * sizeof(gint));

	return;
	} /* put_gimp_saved_values */

void default_global_values (GParam param) {
	/* global_drawable */
	/* global_matrix */
	/* global_center_x */
	/* global_center_y */
	int i, j;

	global_drawable = gimp_drawable_get (param.data.d_drawable);
	global_center_x = 2;
	global_center_y = 2;

	for(i = 0; i < INPUT_MATRIX_SIZE; i++) {
		for(j = 0; j < INPUT_MATRIX_SIZE; j++) {
			global_matrix[i][j] = 0;
			} /* i */
		} /* j */

	global_matrix[2][2] = 1;

	return;
	} /* default_global_values */

gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data) {
	g_print ("delete event occurred\n");
	return(TRUE);
	}

void destroy(GtkWidget *widget, gpointer data) {
	gtk_main_quit();
}

GtkWidget * morph_create_window(void) {
	GtkWidget *window;
	GtkWidget *button;
	GtkWidget *table;
	GtkWidget *vbox;
	GtkWidget *hbox;

	/* Build our big little window, ain't it special */
	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_signal_connect (GTK_OBJECT (window), "delete_event", GTK_SIGNAL_FUNC (destroy), NULL);
	gtk_signal_connect (GTK_OBJECT (window), "destroy", GTK_SIGNAL_FUNC (destroy), NULL);
	gtk_container_set_border_width (GTK_CONTAINER (window), 5);

		/* Create VBox for whole thing... */
		vbox = gtk_vbox_new(FALSE, 10);

			/* Create the table itself */
			table = create_gtk_table();
			gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(table), TRUE, TRUE, 5);
			gtk_widget_show(table);

			hbox = gtk_hbox_new(TRUE, 1);

				/* Create Okay Button */
				button = gtk_button_new_with_label ("Okay");
				gtk_signal_connect (GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC (hello), NULL);
				gtk_signal_connect_object (GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (window));
				gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(button), TRUE, TRUE, 5);
				gtk_widget_show (button);

				/* Create Cancel Button */
				button = gtk_button_new_with_label ("Cancel");
				gtk_signal_connect_object (GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (window));
				gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(button), TRUE, TRUE, 5);
				gtk_widget_show (button);

			gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(hbox), FALSE, FALSE, 5);
			gtk_widget_show(hbox);

		gtk_container_add(GTK_CONTAINER(window), vbox);
		gtk_widget_show(vbox);

	gtk_widget_show (window);

	return(window);
} /* morph_create_window */

GtkWidget * create_gtk_table(void) {
	GtkWidget * table;
	GtkWidget * checkbox;
	GtkWidget * frame;
	GtkWidget * hbox;
	GtkWidget * prevbutton;
	int i, j;

	/* Create the overriding HBox */
	hbox = gtk_hbox_new(TRUE, 4);

		/* Create the Mask Frame */
		frame = gtk_frame_new("Mask");
		table = gtk_table_new(INPUT_MATRIX_SIZE, INPUT_MATRIX_SIZE, TRUE);

		for(i = 0; i < INPUT_MATRIX_SIZE; i++) {
			for(j = 0; j < INPUT_MATRIX_SIZE; j++) {
				checkbox = gtk_toggle_button_new();
				if (global_matrix[i][j]) { 
					gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox), TRUE);
					}
				gtk_signal_connect_object (GTK_OBJECT(checkbox), "toggled", GTK_SIGNAL_FUNC (checkbox_change_value), (gpointer)(&(global_matrix[i][j])));
				gtk_table_attach(GTK_TABLE(table), GTK_WIDGET(checkbox), j, j + 1, i, i + 1, GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 0, 0);
				gtk_widget_show(checkbox);
				} /* j */
			} /* i */

		gtk_container_set_border_width(GTK_CONTAINER(table), 5);
		gtk_widget_set_usize(frame, 100, 105);
		gtk_container_add(GTK_CONTAINER(frame), table);
		gtk_widget_show(table);

		gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(frame), TRUE, TRUE, 5);
		gtk_widget_show(frame);

		/* Create the Ceter Frame thingy */
		frame = gtk_frame_new("Center");
		table = gtk_table_new(INPUT_MATRIX_SIZE, INPUT_MATRIX_SIZE, TRUE);
		prevbutton = NULL;

		for(i = 0; i < INPUT_MATRIX_SIZE; i++) {
			for(j = 0; j < INPUT_MATRIX_SIZE; j++) {
				checkbox = gtk_radio_button_new(prevbutton == NULL ? NULL : gtk_radio_button_group(GTK_RADIO_BUTTON(prevbutton)));
				gtk_signal_connect_object (GTK_OBJECT(checkbox), "toggled", GTK_SIGNAL_FUNC (center_callback), (gpointer)(i * INPUT_MATRIX_SIZE + j)); 
				gtk_table_attach(GTK_TABLE(table), GTK_WIDGET(checkbox), j, j + 1, i, i + 1, GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 0, 0);
				if (i == global_center_x && j == global_center_y)
					gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox), TRUE);
				gtk_widget_show(checkbox);
				prevbutton = checkbox;
				} /* j */
			} /* i */

		gtk_container_set_border_width(GTK_CONTAINER(table), 5);
		gtk_widget_set_usize(frame, 100, 105);
		gtk_container_add(GTK_CONTAINER(frame), table);
		gtk_widget_show(table);

		gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(frame), TRUE, TRUE, 5);
		gtk_widget_show(frame);


	return hbox;
	} /* create_gtk_table */

void center_callback (gpointer data) {

	global_center_x = (int)(data) / INPUT_MATRIX_SIZE;
	global_center_y = (int)(data) % INPUT_MATRIX_SIZE;

	return;
	} /* center_callback */

void checkbox_change_value (gpointer data) {
	*(int *)data = 1 - *(int *)data;
	return;
	} /* checkbox_change_value */

GPixelRgn open_me_up (GDrawable *drawable, int matrix[][], int x, int y) {
	GPixelRgn (* dofunc)(GDrawable *drawable, int Se[][], int Px, int Py);

	dofunc = solve_which_function(PLUG_IN_NAME_ERODE);
	dofunc(drawable, matrix, x, y);

	gimp_drawable_flush(global_drawable);
	gimp_drawable_merge_shadow(global_drawable->id, TRUE);
	gimp_drawable_update(global_drawable->id, origin_x1, origin_y1, (origin_x2 - origin_x1), (origin_y2 - origin_y1));
	gimp_displays_flush();

	dofunc = solve_which_function(PLUG_IN_NAME_DILATE);
	return dofunc(drawable, matrix, x, y);
	} /* open_me_up */

GPixelRgn close_me_down (GDrawable *drawable, int matrix[][], int x, int y) {
	GPixelRgn (* dofunc)(GDrawable *drawable, int Se[][], int Px, int Py);

	dofunc = solve_which_function(PLUG_IN_NAME_DILATE);
	dofunc(drawable, matrix, x, y);

	gimp_drawable_flush(global_drawable);
	gimp_drawable_merge_shadow(global_drawable->id, TRUE);
	gimp_drawable_update(global_drawable->id, origin_x1, origin_y1, (origin_x2 - origin_x1), (origin_y2 - origin_y1));
	gimp_displays_flush();

	dofunc = solve_which_function(PLUG_IN_NAME_ERODE);
	return dofunc(drawable, matrix, x, y);
	} /* open_me_up */
